package org.toilelibre.libe.ratrap.shared;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

public class Path implements Serializable, Comparable<Path> {

    /**
   * 
   */
    private static final long serialVersionUID = 2055728007310596061L;

    private int               cut;
    private Station           end;
    private List<Line>        lineSteps;
    private Station           start;
    private List<Station>     stationSteps;
    private String            type;

    public Path () {
        super ();
        this.cut = 0;
    }

    public Path (final String t) {
        super ();
        this.stationSteps = new ArrayList<Station> ();
        this.lineSteps = new ArrayList<Line> ();
        this.type = t;
        this.cut = 0;
    }

    public void append (final Station sp1, final Line lp1) {
        if (this.cut == 0) {
            this.lineSteps.add (lp1);
            this.stationSteps.add (sp1);
        } else {
            this.lineSteps.add (this.lineSteps.size () - this.cut, lp1);
            this.stationSteps.add (this.stationSteps.size () - this.cut, sp1);
        }
    }

    @Override
    public int compareTo (final Path arg0) {
        if (arg0 == null) {
            return -1;
        }
        return new Double (this.computeDistance ()).compareTo (new Double (arg0.computeDistance ()));
    }

    private double computeDistance () {
        double totalDistance = 0.0;
        Station s = this.start;
        if (this.getStationSteps () == null) {
            return totalDistance;
        }
        for (final Station s1 : this.getStationSteps ()) {
            totalDistance += DistanceCompute.distance (s, s1);
            s = s1;
        }
        totalDistance += DistanceCompute.distance (s, this.end);
        return totalDistance;
    }

    public Station getEnd () {
        return this.end;
    }

    public List<Line> getLineSteps () {
        return this.lineSteps;
    }

    public Station getStart () {
        return this.start;
    }

    public List<Station> getStationSteps () {
        return this.stationSteps;
    }

    public String getType () {
        return this.type;
    }

    public void insert (final Station sp1, final Line lp1) {
        this.lineSteps.add (this.cut, lp1);
        this.stationSteps.add (this.cut++ , sp1);
    }

    public void setEnd (final Station end) {
        this.end = end;
    }

    public void setLineSteps (final List<Line> lineSteps) {
        this.lineSteps = lineSteps;
    }

    public void setStart (final Station start) {
        this.start = start;
    }

    public void setStationSteps (final List<Station> steps) {
        this.stationSteps = steps;
    }

    public void setType (final String type) {
        this.type = type;
    }

    @Override
    public String toString () {
        final StringBuffer sb = new StringBuffer ("Trajet de type " + this.type + " :\n");
        Line l = null;
        if (this.start != null) {
            sb.append (" v d&eacute;part : " + this.start.getName () + "\n");
        }
        for (int i = this.stationSteps.size () - 1 ; i >= 0 ; i-- ) {
            if (l != this.lineSteps.get (i)) {
                l = this.lineSteps.get (i);
                if (i < (this.stationSteps.size () - 1)) {
                    sb.append (" |- Correspondance " + l + " ; ");
                } else {
                    sb.append (" |- " + l + " ; ");
                }
            } else {
                sb.append (" | ");
            }
            sb.append ("Station " + this.stationSteps.get (i).getName () + "\n");
        }
        if (this.end != null) {
            sb.append (" X arriv&eacute;e &agrave; " + this.end.getName () + "\n");
        }
        return sb.toString ();
    }
}
